Engine.DebugLua(true)

local Objects    = {}
local Velocities = {}
local CameraObj        -- will hold our camera
local CamSpeed   = 800 -- camera speed in world units/sec

-- new border extents
local MinX, MinY = 0, 0
local MaxX, MaxY = 5096, 5096
local Speed      = 1000 -- light speed in units per second

-- simple clamp
local function clamp(v, lo, hi)
    if v < lo then return lo end
    if v > hi then return hi end
    return v
end

function OnInit()
    math.randomseed(os.time())
    Engine.LogInfo("Initial Speed: " .. Speed)

    -- cache lights
    for i = 0, 99 do
        local tag = "Light_" .. i
        local obj = Engine.GetObjectByTag(tag)
        if obj then
            table.insert(Objects, obj)
            local ang       = math.random() * 2 * math.pi
            local vx        = math.cos(ang) * Speed
            local vy        = math.sin(ang) * Speed
            Velocities[obj] = { x = vx, y = vy }
        else
            Engine.LogError("Object '" .. tag .. "' not found!")
        end
    end

    -- grab the camera
    CameraObj = Engine.GetObjectByTag("Camera")
    if not CameraObj then
        Engine.LogError("Camera object not found!")
    end
end

function OnUpdate(dt)
    -- light speed tweaks (same as before)
    if Engine.KeyDown(Keycode.Equal) or Engine.KeyDown(Keycode.KPAdd) then
        Speed = Speed + 100 * dt
        Engine.LogInfo(string.format("Speed increased to %.1f", Speed))
    elseif Engine.KeyDown(Keycode.Minus) or Engine.KeyDown(Keycode.KPSubtract) then
        Speed = Speed - 100 * dt
        if Speed < 0 then Speed = 0 end
        Engine.LogInfo(string.format("Speed decreased to %.1f", Speed))
    end

    
    if not CameraObj then
        Engine.LogError("Camera object not found!")
        CameraObj = Engine.GetObjectByTag("Camera")
    end

    -- move lights
    for _, obj in ipairs(Objects) do
        local vel = Velocities[obj]
        local pos = obj:GetPosition()

        local ang = math.atan(vel.y, vel.x)
        vel.x = math.cos(ang) * Speed
        vel.y = math.sin(ang) * Speed

        local nx = pos.x + vel.x * dt
        local ny = pos.y + vel.y * dt

        if nx < MinX or nx > MaxX then
            vel.x = -vel.x
            nx    = clamp(nx, MinX, MaxX)
        end
        if ny < MinY or ny > MaxY then
            vel.y = -vel.y
            ny    = clamp(ny, MinY, MaxY)
        end

        Velocities[obj] = vel
        obj:SetPosition(Vector2(nx, ny))
    end

    if CameraObj then
        local camPos = CameraObj:GetPosition()
        if Engine.KeyDown(Keycode.W) then
            camPos.y = camPos.y - CamSpeed * dt
        elseif Engine.KeyDown(Keycode.S) then
            camPos.y = camPos.y + CamSpeed * dt
        end
        if Engine.KeyDown(Keycode.A) then
            camPos.x = camPos.x - CamSpeed * dt
        elseif Engine.KeyDown(Keycode.D) then
            camPos.x = camPos.x + CamSpeed * dt
        end
        CameraObj:SetPosition(Vector2(camPos.x, camPos.y))
    end
end
