// core/render/AssetBrowser.cpp
#include "AssetBrowser.h"
#include <imgui.h>

extern std::unordered_map<std::string, ImageCacheEntry> textureCache;

void ShowAssetBrowser()
{
    PROFILE_ENGINE_SCOPE("Editor::AssetBrowser");

    ImGui::Begin("Asset Browser");

    if (ImGui::Button("Load Asset"))
    {
        std::string path = OpenFileDialog(FileDialogType::Images);
        if (!path.empty())
        {
            LoadTextureIfNeeded(path);
        }
    }

    ImGui::Separator();

    ImGui::BeginChild("##AssetScroll", ImVec2(0, 0), true);

    const float padding       = 8.0f;
    const float thumbnailSize = 64.0f;
    const float cellSize      = thumbnailSize + padding;

    float availWidth = ImGui::GetContentRegionAvail().x;
    int   columns    = static_cast<int>(availWidth / cellSize);
    if (columns < 1) columns = 1;

    ImGui::Columns(columns, nullptr, false);

    int idx = 0;
    for (auto& kv : textureCache)
    {
        ImGui::PushID(idx++);
        const std::string& path  = kv.first;
        const auto&        entry = kv.second;

        std::string displayName = GetFilenameFromPath(path);

        ImGui::ImageButton("##Image",
            (ImTextureID)(intptr_t)entry.textureID,
            ImVec2(thumbnailSize, thumbnailSize),
            ImVec2(0,0), ImVec2(1,1)
        );

        if (ImGui::BeginDragDropSource())
        {
            ImGui::SetDragDropPayload("ASSET_TEXTURE",
                                      path.c_str(),
                                      path.size()+1);
            ImGui::Image((ImTextureID)(intptr_t)entry.textureID,
                         ImVec2(thumbnailSize*2, thumbnailSize*2));
            ImGui::Text("%s", displayName.c_str());
            ImGui::EndDragDropSource();
        }

        ImGui::TextWrapped("%s", displayName.c_str());
        ImGui::NextColumn();
        ImGui::PopID();
    }

    ImGui::Columns(1);
    ImGui::EndChild();
    ImGui::End();
}
